#define AMXMODX
//#define AMXMOD

/**
 * Notes - 
 * 1] AMX Mod 0.9.9 uses Small 2.6.0, which can't detect stack sizes properly.
 *     Because of this, StringBench() fails allocating 4096 cells on the
 *     stack with AMX_ERR_STACKERR.  This is fixed by manually using 
 *     #pragma dynamic.
 * 2] AMX Mod cannot directly translate a string to a given language.
 *     This is not necessarily a problem (rather than a missing feature),
 *     but it means we can't benchmark random language lookup.  For this
 *     demonstration, I use the server language and assume it's "en".
 * 3] AMX Mod X has "formatex()" as a replacement for "format()".  
 *     It eliminates an input/output overlap check for coders who know
 *     they are using format() with non-overlapping strings.  Since this
 *     script does not use overlapping buffers, I mapped format -> formatex
 *     for a small speed gain (~.5s)
 */ 

#if defined AMXMODX
#include <amxmodx>
//see (3)
#define format formatex
#else
#pragma dynamic 32768	//see (1)
#include <translator>	//this must come before <amxmod>
#include <amxmod>
#endif

#define MATH_INT_LOOPS		10000000
#define MATH_FLOAT_LOOPS	10000000
#define STRING_FMT_LOOPS	2000000
#define STRING_ML_LOOPS		2000000

new g_lang_time

native get_tick_count();

public plugin_init()
{
	register_plugin("BenchMark", "2.0", "AMXx Dev Team");
	register_srvcmd("bench", "BenchBegin");
	new start = systime();
	new end;
#if defined AMXMODX
	register_dictionary("fakedict-amxmodx.txt");
#else
	load_translations("fakedict-amxmod");
#endif
	end = systime();
	g_lang_time = end - start;
}

public BenchBegin()
{
	server_print("dictionary time: %d milliseconds", g_lang_time);
	StringBench();
	MathBench();
}
stock systime()
{
	return get_tick_count();
}

MathBench()
{
	new iter = MATH_INT_LOOPS;
	new start = get_tick_count();
	new end;
	new a, b, c;
	while(iter--)
	{
		a = iter * 7;
		b = 5 + iter;
		c = 6 / (iter + 3);
		a = 6 * (iter);
		b = a * 185;
		a = b / 25;
		c = b - a + 3;
		b = b*b;
		a = (a + c) / (b - c);
		b = 6;
		c = 1;
		b = a * 128 - c;
		c = b * (a + 16) * b;
		if (!a)
			a = 5;
		a = c + (28/a) - c;
	}
	end = get_tick_count();
	server_print("int benchmark: %d milliseconds", end-start);
	
	new Float:fa, Float:fb, Float:fc
	new int1
	iter = MATH_FLOAT_LOOPS;
	start = get_tick_count();
	while (iter--)
	{
		fa = iter * 0.7;
		fb = 5.1 + iter;
		fc = 6.1 / (float(iter) + 2.5);
		fa = 6.1 * (iter);
		fb = fa * 185.26;
		fa = fb / 25.56;
		fc = fb - a + float(3);
		fb = fb*fb;
		fa = (fa + fc) / (fb - fc);
		fb = 6.2;
		fc = float(1);
		int1 = floatround(fa);
		fb = fa * float(128) - int1;
		fc = fb * (a + 16.85) * float(floatround(fb, floatround_ceil));
		if (fa == 0.0)
			fa = 5.0;
		fa = fc + (float(28)/fa) - floatround(fc, floatround_floor);
	}
	end = get_tick_count();
	server_print("float benchmark: %d milliseconds", end-start);
}

/**
 * These keys should exist in the dictionary file!
 * In the base setup, keys 1/2 are from middle
 * keys 3/4 are from beginning
 * keys 5/6 are from end
 * Also, see note (2)
 */
#define KEY1	"KQDXJJWUH"
#define KEY2	"SOY"
#define KEY3	"QWOZNKWPXWKC"
#define KEY4	"IVCBQXXKTAY"
#define KEY5	"MSRT"
#define KEY6	"LJHAZVAEEKKF"

StringBench()
{
	new i = STRING_FMT_LOOPS;
	new buffer[255];
	
	new start = systime();
	new end;
	while (i--)
	{
		end = 0;
		format(buffer, 254, "%d", i);
		format(buffer, 254, "%d %s %d %f %d %-3.4s %s", i, "gaben", 30, 10.0, 20, "hello", "What a gaben");
		end = format(buffer, 254, "Well, that's just %-17.18s!", "what.  this isn't a valid string! wait it is");
		end += format(buffer[end], 254-end, "There are %d in this %d", i, end);
		end += format(buffer[end], 254-end, "There are %d in this %d", i, end);
	}
	end = systime();
	server_print("format() benchmark: %d milliseconds", end-start);
	
	start = get_tick_count();
	i = STRING_ML_LOOPS;
	new fmtbuf[2048];
	while (i--)
	{
#if defined AMXMODX
		format(fmtbuf, 2047, "%L %L %d %s %f %L", "en", KEY1, "en", KEY2, 50, "what the", 50.0, "en", KEY3);
		format(fmtbuf, 2047, "%s %L %s %L %L", "gaben", "en", KEY4, "what TIME is it", "en", KEY5, "en", KEY6);
#else
		format(fmtbuf, 2047, "%s %s %d %s %f %s", _T(KEY1,0), _T(KEY2,0), 50, "what the", 50.0, _T(KEY3,0));
		format(fmtbuf, 2047, "%s %s %s %s %s", "gaben", _T(KEY4,0), "what TIME is it", _T(KEY5,0), _T(KEY6,0));
#endif
	}
	end = get_tick_count();
	server_print("ml benchmark: %d milliseconds", end-start);
}
